/*
	
	Step 3 of 3, Object To Volume Process (ToVolume)
	IsoRender.pov
	2013 Sam Benge
	
	This scene file demonstrates how a series of image files can be compiled into a volumetric pattern, and rendered using only POV-Ray's
	SDL. The idea is to place a series of frames into a pattern container (a gradient z pigment), and then render the result as an isosurface
	or other volumetric representation.
	
	Check out the various settings to use, below.
	
*/


#version 3.7;

#macro Lerp(A, B, C) A*(1-C)+B*C #end

global_settings{
	assumed_gamma 1.0
	#if(1)
		radiosity{
			count 64
			error_bound .1
			recursion_limit 1
			nearest_count 1
			normal on
			pretrace_end .04/3
			
			//media on
		}
	#end
	max_trace_level 3
}

#default{ finish{ambient 0} }//normal{bumps .125 scale .001} }

#declare SunPos = vrotate(y, <-23, 90, 0>);
#declare RGBSun = srgb 0.625*<2, 1.95, 1.8>;
#declare RGBSky = rgb <.2, .3, .75>;

#declare CamPos = .175*<0, 0, -35>;
camera{
	location vrotate(CamPos, <35, 3, 0>)
	look_at 0
	angle 30
}

// a faraway light_source
#if(1)
	light_source{
		SunPos*1e7, RGBSun
		
		photons{area_light}
		#if(1)
			#declare ALRes	= 5;
			#declare ALSize	= 0.03*1e7;
			area_light x*ALSize, z*ALSize, ALRes, ALRes
			jitter
			adaptive 1
		#end
	}
#end

// blue sky...
#if(1)
	sky_sphere{
		pigment{
			spherical
			translate vnormalize(SunPos)
			poly_wave 2
			color_map{
				[0 RGBSky]
				[1 RGBSun]
			}
		}
	}
#end


//*** some important settings ***//

// render either an isosurface or a mesh
#declare RenderIso = 1;

// size of the object, given by RegisterObject.pov
#declare Size = <648, 541, 256>;

//*** end important settings ***//


#declare NFrames	= Size.z; // number of frames, also used to determine zero padding in Image_Name, below
#declare Size = Size / Size.z; // the final Size is scaled using its own z component

// ground plane
#if(1)
	
	plane{
		y, -Size.z/2 // set plane to bottom of object (change to Size.y if object below is not rotated)
		pigment{
			bumps
			scale 3
			warp{turbulence 1 lambda 3}
			color_map{[0 rgb .1][1 rgb .5]}
		}
		normal{bumps .25 scale .001}
	}
	
#end // ground plane

#if(1) // render either object
  
  #if(RenderIso) // isosurface
		
		// importnant: base name to use when loading image slices
		#declare Image_Name_ = "Slice"
		
		// create volume
		#declare Volume =
			pigment{
				gradient z
				pigment_map{
					
					#for(V, 1, NFrames)
						[V/NFrames
							#declare Image_Name = concat(Image_Name_, str(V, -(log(NFrames)/log(10) + 1), 0))
							image_map{png Image_Name interpolate 2 }//once}
						]
					#end
				}
				// center volume
				translate -1/2
				
				// scale volume to Size
				scale Size
			}
		// ~create volume
		
		#declare FVolume = function{pattern{pigment_pattern{Volume}}}
		
		// the isosurface
		isosurface{
			function{
				FVolume(x, y, z) - 0.5
			}
			accuracy .001
			max_gradient 50
			contained_by{box{-Size/2+1e-7, Size/2-1e-7}}
			
			pigment{
				rgb .75
			}
			
			// only necessary if a rotation was used in the previous files
			rotate x*90
			
		} // iso
	
	#else
		
		// Mesh
		
		#include "bowl_POV_geom.inc" // bowl_mesh_
	  
	  #declare Obj =
			object{
				bowl_mesh_
			}
		
		#declare Obj2 =
			object{
				Obj
				translate -min_extent(Obj)
			}
		
		#declare Obj3 =
			object{
				Obj2
				scale 1/max_extent(Obj).z
			}	
	  
		object{
			Obj3
			translate -max_extent(Obj3)/2
			
			pigment{rgb .75
			
		}
	#end
#end